#include "imgui_settings_dialog.h"
#include "plugin.h"
#include "imgui/backends/imgui_impl_win32.h"
#include "imgui/backends/imgui_impl_dx9.h"
#include <d3d9.h>
#include <tchar.h>
#include <string>
#include <commdlg.h>
#include <algorithm> // For std::min and std::max

// Global variables for ImGui and DirectX
static HWND g_hWnd = NULL;
static HWND g_hWndParent = NULL;
static IDirect3D9 *g_pD3D = NULL;
static IDirect3DDevice9 *g_pd3dDevice = NULL;
static D3DPRESENT_PARAMETERS g_d3dpp = {};
static bool g_dialogOpen = false;
static LogOptions g_currentOptions;
static std::string g_currentOutputFile;
static D3DCOLOR g_clearColor = D3DCOLOR_RGBA(245, 245, 250, 255); // Глобальный цвет фона для всех функций

// Global variables for help window
static bool g_HelpWindowOpen = false;
static HWND g_HelpWindowParent = NULL;
static HWND g_HelpWindow = NULL;
static WNDPROC g_HelpOriginalProc = NULL;
static IDirect3D9 *g_pHelpD3D = NULL;
static IDirect3DDevice9 *g_pdHelpD3DDevice = NULL;
static D3DPRESENT_PARAMETERS g_helpD3dpp = {};

// Forward declarations
bool CreateDeviceD3D(HWND hWnd);
void CleanupDeviceD3D();
void ResetDevice();
LRESULT WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

// Initialize ImGui for the settings dialog
bool InitImGuiSettingsDialog()
{
  // Create application window
  WNDCLASSEXW wc = {sizeof(wc), CS_CLASSDC, WndProc, 0L, 0L, GetModuleHandle(NULL), NULL, NULL, NULL, NULL, L"ImGui Settings Dialog", NULL};
  RegisterClassExW(&wc);

  // Window size and positioning
  int width = 600;
  int height = 500;
  int x = (GetSystemMetrics(SM_CXSCREEN) - width) / 2;
  int y = (GetSystemMetrics(SM_CYSCREEN) - height) / 2;

  // If parent window exists, center on parent
  if (g_hWndParent)
  {
    RECT rcParent;
    GetWindowRect(g_hWndParent, &rcParent);
    x = rcParent.left + ((rcParent.right - rcParent.left) - width) / 2;
    y = rcParent.top + ((rcParent.bottom - rcParent.top) - height) / 2;

    // Make sure dialog stays on screen
    x = std::max(0, std::min(x, GetSystemMetrics(SM_CXSCREEN) - width));
    y = std::max(0, std::min(y, GetSystemMetrics(SM_CYSCREEN) - height));

    // Disable parent window to create modal-like behavior
    EnableWindow(g_hWndParent, FALSE);
  }

  g_hWnd = CreateWindowW(wc.lpszClassName, L"✧ Instruction Logger Settings ✧",
                         WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX,
                         x, y, width, height,
                         g_hWndParent, NULL, wc.hInstance, NULL);

  // Set window to be topmost so it stays above parent
  SetWindowPos(g_hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);

  // Initialize Direct3D
  if (!CreateDeviceD3D(g_hWnd))
  {
    CleanupDeviceD3D();
    UnregisterClassW(wc.lpszClassName, wc.hInstance);
    if (g_hWndParent)
      EnableWindow(g_hWndParent, TRUE);
    return false;
  }

  // Show the window
  ShowWindow(g_hWnd, SW_SHOWDEFAULT);
  UpdateWindow(g_hWnd);

  // Setup ImGui context
  IMGUI_CHECKVERSION();
  ImGui::CreateContext();
  ImGuiIO &io = ImGui::GetIO();
  io.ConfigFlags |= ImGuiConfigFlags_NavEnableKeyboard; // Enable Keyboard Controls

  // Setup angel-like light ImGui style
  ImGui::StyleColorsLight();
  ImGuiStyle& style = ImGui::GetStyle();

  // Светлая ангельская цветовая гамма
  ImVec4* colors = style.Colors;
  colors[ImGuiCol_Text]                   = ImVec4(0.20f, 0.20f, 0.30f, 1.00f);
  colors[ImGuiCol_TextDisabled]           = ImVec4(0.60f, 0.60f, 0.70f, 1.00f);
  colors[ImGuiCol_WindowBg]               = ImVec4(0.96f, 0.96f, 0.99f, 1.00f);
  colors[ImGuiCol_ChildBg]                = ImVec4(0.92f, 0.94f, 0.98f, 1.00f);
  colors[ImGuiCol_PopupBg]                = ImVec4(0.95f, 0.95f, 0.98f, 0.98f);
  colors[ImGuiCol_Border]                 = ImVec4(0.80f, 0.82f, 0.90f, 0.50f);
  colors[ImGuiCol_BorderShadow]           = ImVec4(0.00f, 0.00f, 0.00f, 0.00f);
  colors[ImGuiCol_FrameBg]                = ImVec4(0.85f, 0.87f, 0.95f, 1.00f);
  colors[ImGuiCol_FrameBgHovered]         = ImVec4(0.75f, 0.80f, 0.93f, 0.60f);
  colors[ImGuiCol_FrameBgActive]          = ImVec4(0.70f, 0.75f, 0.88f, 0.80f);
  colors[ImGuiCol_TitleBg]                = ImVec4(0.75f, 0.80f, 0.95f, 1.00f);
  colors[ImGuiCol_TitleBgActive]          = ImVec4(0.70f, 0.78f, 0.94f, 1.00f);
  colors[ImGuiCol_TitleBgCollapsed]       = ImVec4(0.80f, 0.85f, 0.95f, 0.75f);
  colors[ImGuiCol_MenuBarBg]              = ImVec4(0.85f, 0.87f, 0.97f, 1.00f);
  colors[ImGuiCol_ScrollbarBg]            = ImVec4(0.90f, 0.92f, 0.97f, 1.00f);
  colors[ImGuiCol_ScrollbarGrab]          = ImVec4(0.70f, 0.75f, 0.88f, 0.80f);
  colors[ImGuiCol_ScrollbarGrabHovered]   = ImVec4(0.60f, 0.68f, 0.85f, 0.80f);
  colors[ImGuiCol_ScrollbarGrabActive]    = ImVec4(0.55f, 0.62f, 0.83f, 1.00f);
  colors[ImGuiCol_CheckMark]              = ImVec4(0.50f, 0.60f, 0.85f, 1.00f);
  colors[ImGuiCol_SliderGrab]             = ImVec4(0.60f, 0.70f, 0.90f, 0.80f);
  colors[ImGuiCol_SliderGrabActive]       = ImVec4(0.50f, 0.60f, 0.85f, 1.00f);
  colors[ImGuiCol_Button]                 = ImVec4(0.75f, 0.80f, 0.93f, 0.80f);
  colors[ImGuiCol_ButtonHovered]          = ImVec4(0.65f, 0.73f, 0.90f, 1.00f);
  colors[ImGuiCol_ButtonActive]           = ImVec4(0.55f, 0.65f, 0.88f, 1.00f);
  colors[ImGuiCol_Header]                 = ImVec4(0.70f, 0.78f, 0.94f, 0.60f);
  colors[ImGuiCol_HeaderHovered]          = ImVec4(0.60f, 0.70f, 0.90f, 0.80f);
  colors[ImGuiCol_HeaderActive]           = ImVec4(0.55f, 0.65f, 0.88f, 1.00f);
  colors[ImGuiCol_Separator]              = ImVec4(0.75f, 0.80f, 0.90f, 0.50f);
  colors[ImGuiCol_SeparatorHovered]       = ImVec4(0.65f, 0.70f, 0.85f, 0.70f);
  colors[ImGuiCol_SeparatorActive]        = ImVec4(0.55f, 0.65f, 0.85f, 0.90f);
  colors[ImGuiCol_ResizeGrip]             = ImVec4(0.60f, 0.70f, 0.90f, 0.25f);
  colors[ImGuiCol_ResizeGripHovered]      = ImVec4(0.50f, 0.60f, 0.85f, 0.67f);
  colors[ImGuiCol_ResizeGripActive]       = ImVec4(0.45f, 0.55f, 0.80f, 0.95f);
  colors[ImGuiCol_Tab]                    = ImVec4(0.75f, 0.80f, 0.93f, 0.80f);
  colors[ImGuiCol_TabHovered]             = ImVec4(0.60f, 0.70f, 0.90f, 1.00f);
  colors[ImGuiCol_TabActive]              = ImVec4(0.68f, 0.75f, 0.92f, 1.00f);
  colors[ImGuiCol_TabUnfocused]           = ImVec4(0.80f, 0.85f, 0.95f, 0.97f);
  colors[ImGuiCol_TabUnfocusedActive]     = ImVec4(0.75f, 0.80f, 0.93f, 1.00f);
  colors[ImGuiCol_PlotLines]              = ImVec4(0.40f, 0.45f, 0.70f, 1.00f);
  colors[ImGuiCol_PlotLinesHovered]       = ImVec4(0.30f, 0.35f, 0.60f, 1.00f);
  colors[ImGuiCol_PlotHistogram]          = ImVec4(0.40f, 0.45f, 0.70f, 1.00f);
  colors[ImGuiCol_PlotHistogramHovered]   = ImVec4(0.30f, 0.35f, 0.60f, 1.00f);
  colors[ImGuiCol_TextSelectedBg]         = ImVec4(0.70f, 0.75f, 0.88f, 0.50f);
  colors[ImGuiCol_DragDropTarget]         = ImVec4(0.65f, 0.70f, 0.85f, 0.90f);
  colors[ImGuiCol_NavHighlight]           = ImVec4(0.70f, 0.80f, 0.95f, 1.00f);
  colors[ImGuiCol_NavWindowingHighlight]  = ImVec4(0.60f, 0.70f, 0.90f, 1.00f);
  colors[ImGuiCol_NavWindowingDimBg]      = ImVec4(0.20f, 0.25f, 0.35f, 0.50f);
  colors[ImGuiCol_ModalWindowDimBg]       = ImVec4(0.20f, 0.25f, 0.35f, 0.35f);

  // Изменение стиля элементов для более мягкого, ангельского вида
  style.WindowRounding = 8.0f;
  style.ChildRounding = 4.0f;
  style.FrameRounding = 6.0f;
  style.PopupRounding = 6.0f;
  style.ScrollbarRounding = 6.0f;
  style.GrabRounding = 6.0f;
  style.TabRounding = 6.0f;
  style.FrameBorderSize = 0.0f;
  style.WindowBorderSize = 1.0f;
  style.PopupBorderSize = 1.0f;
  style.ItemSpacing = ImVec2(8, 6);
  style.ItemInnerSpacing = ImVec2(6, 6);
  style.FramePadding = ImVec2(6, 3);
  style.WindowPadding = ImVec2(12, 12);
  style.Alpha = 1.0f;

  // Увеличим размер шрифта для заголовков и кнопок
  io.FontGlobalScale = 1.1f;

  // Использование скругленных краев для всех элементов
  style.WindowTitleAlign = ImVec2(0.5f, 0.5f); // Центрирование заголовка окна
  style.ButtonTextAlign = ImVec2(0.5f, 0.5f);  // Центрирование текста на кнопках

  // Изменим цвет фона окна на светлый (уже определен как глобальная переменная g_clearColor)
  // static D3DCOLOR clearColor = D3DCOLOR_RGBA(245, 245, 250, 255);

  // Setup Platform/Renderer backends
  ImGui_ImplWin32_Init(g_hWnd);
  ImGui_ImplDX9_Init(g_pd3dDevice);

  // Get current options
  if (g_instrLogger)
  {
    g_currentOptions = g_instrLogger->GetOptions();
    g_currentOutputFile = g_currentOptions.outputFile;
  }

  g_dialogOpen = true;
  return true;
}

// Shutdown ImGui
void ShutdownImGuiSettingsDialog()
{
    DebugLog("[%s] Shutting down ImGui settings dialog\n", PLUGIN_NAME);

    // Cleanup ImGui
    if (ImGui::GetCurrentContext())
    {
        ImGui_ImplDX9_Shutdown();
        ImGui_ImplWin32_Shutdown();
        ImGui::DestroyContext();
    }

    // Cleanup DirectX
    CleanupDeviceD3D();

    // Re-enable parent window if still exists
    if (g_hWndParent && IsWindow(g_hWndParent))
    {
        EnableWindow(g_hWndParent, TRUE);
        SetForegroundWindow(g_hWndParent);
    }

    // Destroy the window if it still exists
    if (g_hWnd && IsWindow(g_hWnd))
    {
        DestroyWindow(g_hWnd);
    }

    // Unregister the window class
    UnregisterClassW(L"ImGui Settings Dialog", GetModuleHandle(NULL));

    g_hWnd = NULL;
    g_dialogOpen = false;

    DebugLog("[%s] ImGui resources cleaned up\n", PLUGIN_NAME);
}

// Show the settings dialog using ImGui
void ShowImGuiSettingsDialog(HWND hWndParent)
{
  // Early exit if dialog is already open
  if (g_dialogOpen)
    return;

  // Store parent window
  g_hWndParent = hWndParent;

  // Reload settings from INI
  if (g_instrLogger)
  {
    DebugLog("[%s] Reloading settings from INI file before showing dialog\n", PLUGIN_NAME);
    g_instrLogger->ReloadSettings();
  }

  // Initialize dialog
  if (!InitImGuiSettingsDialog())
  {
    DebugLog("[%s] Failed to initialize ImGui settings dialog\n", PLUGIN_NAME);
    return;
  }

  // Message loop - don't use WM_QUIT since it can affect the parent application
  MSG msg;
  ZeroMemory(&msg, sizeof(msg));
  while (g_dialogOpen)
  {
    // Use PeekMessage instead of GetMessage to avoid blocking
    while (PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE))
    {
      // Filter out WM_QUIT messages to prevent closing the parent
      if (msg.message == WM_QUIT)
      {
        DebugLog("[%s] WM_QUIT message intercepted and discarded\n", PLUGIN_NAME);
        g_dialogOpen = false;
        break;
      }

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

    // Exit the message loop if dialog is no longer open
    if (!g_dialogOpen)
      break;

    // Render ImGui frame
    RenderImGuiSettingsDialog();
  }

  // Restore parent window focus and enable it
  if (g_hWndParent)
  {
    EnableWindow(g_hWndParent, TRUE);
    SetForegroundWindow(g_hWndParent);
  }

  // Shutdown
  ShutdownImGuiSettingsDialog();

  DebugLog("[%s] ImGui settings dialog closed\n", PLUGIN_NAME);
}

// Helper function to browse for a file
bool BrowseForOutputFile(std::string &filePath)
{
  char szFile[MAX_PATH] = {0};
  if (!filePath.empty() && filePath.length() < MAX_PATH)
    strcpy_s(szFile, filePath.c_str());

  OPENFILENAMEA ofn = {0};
  ofn.lStructSize = sizeof(ofn);
  ofn.hwndOwner = g_hWnd;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFilter = "CSV Files (*.csv)\0*.csv\0Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
  ofn.nFilterIndex = 1;
  ofn.lpstrDefExt = "csv";
  ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT;

  if (GetSaveFileNameA(&ofn))
  {
    filePath = szFile;
    return true;
  }
  return false;
}

// Render a frame of the settings dialog
void RenderImGuiSettingsDialog()
{
  // Start the Dear ImGui frame
  ImGui_ImplDX9_NewFrame();
  ImGui_ImplWin32_NewFrame();
  ImGui::NewFrame();

  // Create Settings Window (always full size)
  ImGui::SetNextWindowPos(ImVec2(0, 0));
  ImGui::SetNextWindowSize(ImVec2((float)g_d3dpp.BackBufferWidth, (float)g_d3dpp.BackBufferHeight));
  ImGuiWindowFlags window_flags = ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoCollapse |
                                  ImGuiWindowFlags_NoResize | ImGuiWindowFlags_NoMove |
                                  ImGuiWindowFlags_NoScrollbar | ImGuiWindowFlags_NoScrollWithMouse;

  ImGui::Begin("Settings", NULL, window_flags);

  // Title с более красивым оформлением
  ImGui::PushFont(ImGui::GetIO().Fonts->Fonts[0]);
  ImGui::Spacing();
  ImGui::SetCursorPosX((ImGui::GetWindowWidth() - ImGui::CalcTextSize("Instruction Logger Settings").x) * 0.5f);
  ImGui::TextColored(ImVec4(0.40f, 0.50f, 0.80f, 1.00f), "Instruction Logger Settings");
  ImGui::PopFont();
  ImGui::Spacing();
  ImGui::Separator();
  ImGui::Spacing();

  // Instruction types to log
  ImGui::TextColored(ImVec4(0.45f, 0.55f, 0.85f, 1.00f), "Instruction Types to Log");
  ImGui::Checkbox("CALL instructions", &g_currentOptions.logCalls);
  ImGui::Checkbox("JMP instructions", &g_currentOptions.logJumps);
  ImGui::Checkbox("RET instructions", &g_currentOptions.logReturns);

  ImGui::Spacing();
  ImGui::Separator();
  ImGui::Spacing();

  // Logging options
  ImGui::TextColored(ImVec4(0.45f, 0.55f, 0.85f, 1.00f), "Logging Options");
  ImGui::Checkbox("Log only current module", &g_currentOptions.logOnlyCurrentModule);
  ImGui::Checkbox("Follow in CPU", &g_currentOptions.followInCpu);

  ImGui::Spacing();
  ImGui::Separator();
  ImGui::Spacing();

  // File output options
  ImGui::TextColored(ImVec4(0.45f, 0.55f, 0.85f, 1.00f), "File Output Options");

  // Log to file checkbox
  if (ImGui::Checkbox("Log to file", &g_currentOptions.logToFile))
  {
    // If we just disabled file logging, hide the file path controls
    if (!g_currentOptions.logToFile)
      g_currentOptions.outputFile = "";
  }

  // Output file path
  if (g_currentOptions.logToFile)
  {
    ImGui::BeginGroup();

    // Limit the display size of the path but keep the full path for editing
    char buffer[MAX_PATH];
    strcpy_s(buffer, g_currentOutputFile.c_str());

    ImGui::Text("Output file:");
    ImGui::SameLine();

    // Calculate available width for the input field
    float availWidth = ImGui::GetContentRegionAvail().x - 70.0f; // Save space for the button

    ImGui::PushItemWidth(availWidth);
    if (ImGui::InputText("##OutputFile", buffer, MAX_PATH))
    {
      g_currentOutputFile = buffer;
    }
    ImGui::PopItemWidth();

    ImGui::SameLine();
    if (ImGui::Button("Browse..."))
    {
      if (BrowseForOutputFile(g_currentOutputFile))
      {
        // Update the options with the new path
        g_currentOptions.outputFile = g_currentOutputFile;
      }
    }

    ImGui::EndGroup();
  }

  ImGui::Spacing();
  ImGui::Separator();
  ImGui::Spacing();

  // Trace settings
  ImGui::TextColored(ImVec4(0.45f, 0.55f, 0.85f, 1.00f), "Trace Settings");

  // Trace step count (with min/max limits)
  int step_count = g_currentOptions.traceStepCount;
  if (ImGui::SliderInt("Maximum trace steps", &step_count, 1, 10000000, "%d", ImGuiSliderFlags_Logarithmic))
  {
    // Enforce limits
    if (step_count < 1)
      step_count = 1;
    if (step_count > 100000000)
      step_count = 100000000;
    g_currentOptions.traceStepCount = step_count;
  }

  ImGui::Spacing();
  ImGui::Separator();
  ImGui::Spacing();

  // Buttons at the bottom
  ImGui::SetCursorPosY(ImGui::GetWindowHeight() - 50.0f);
  ImGui::Separator();
  ImGui::SetCursorPosY(ImGui::GetWindowHeight() - 40.0f);

  // Save output path from input field to options
  if (g_currentOptions.logToFile)
    g_currentOptions.outputFile = g_currentOutputFile;

  // Обновим стили кнопок для ангельской темы
  // Apply button
  ImVec4 applyColor = ImVec4(0.50f, 0.65f, 0.85f, 1.00f);
  ImVec4 saveColor = ImVec4(0.40f, 0.55f, 0.80f, 1.00f);
  ImVec4 cancelColor = ImVec4(0.70f, 0.70f, 0.80f, 1.00f);

  ImGui::PushStyleColor(ImGuiCol_Button, applyColor);
  ImGui::PushStyleColor(ImGuiCol_ButtonHovered, ImVec4(applyColor.x + 0.1f, applyColor.y + 0.1f, applyColor.z + 0.1f, 1.0f));
  ImGui::PushStyleColor(ImGuiCol_ButtonActive, ImVec4(applyColor.x - 0.1f, applyColor.y - 0.1f, applyColor.z - 0.1f, 1.0f));
  if (ImGui::Button("Apply", ImVec2(120, 30)))
  {
    // Save settings but don't close dialog
    if (g_instrLogger)
    {
      g_instrLogger->SetOptions(g_currentOptions);
      bool saved = g_instrLogger->SaveSettings();
      if (saved)
      {
        DebugLog("[%s] Applied settings in ImGui dialog\n", PLUGIN_NAME);
        _plugin_logprintf("[%s] Settings updated and saved to INI file\n", PLUGIN_NAME);
      }
      else
      {
        DebugLog("[%s] Failed to save settings in ImGui dialog\n", PLUGIN_NAME);
        _plugin_logprintf("[%s] WARNING: Failed to save settings to INI file\n", PLUGIN_NAME);
      }
    }
  }
  ImGui::PopStyleColor(3);

  ImGui::SameLine();

  // Save button
  ImGui::PushStyleColor(ImGuiCol_Button, saveColor);
  ImGui::PushStyleColor(ImGuiCol_ButtonHovered, ImVec4(saveColor.x + 0.1f, saveColor.y + 0.1f, saveColor.z + 0.1f, 1.0f));
  ImGui::PushStyleColor(ImGuiCol_ButtonActive, ImVec4(saveColor.x - 0.1f, saveColor.y - 0.1f, saveColor.z - 0.1f, 1.0f));
  if (ImGui::Button("Save", ImVec2(120, 30)))
  {
    // Save settings and close dialog
    if (g_instrLogger)
    {
      g_instrLogger->SetOptions(g_currentOptions);
      bool saved = g_instrLogger->SaveSettings();
      if (saved)
      {
        DebugLog("[%s] Saved settings in ImGui dialog\n", PLUGIN_NAME);
        _plugin_logprintf("[%s] Settings updated and saved to INI file\n", PLUGIN_NAME);
      }
    }

    // Close the dialog (don't destroy window here, just mark as closed)
    g_dialogOpen = false;
  }
  ImGui::PopStyleColor(3);

  ImGui::SameLine();

  // Cancel button
  ImGui::PushStyleColor(ImGuiCol_Button, cancelColor);
  ImGui::PushStyleColor(ImGuiCol_ButtonHovered, ImVec4(cancelColor.x + 0.1f, cancelColor.y + 0.1f, cancelColor.z + 0.1f, 1.0f));
  ImGui::PushStyleColor(ImGuiCol_ButtonActive, ImVec4(cancelColor.x - 0.1f, cancelColor.y - 0.1f, cancelColor.z - 0.1f, 1.0f));
  if (ImGui::Button("Cancel", ImVec2(120, 30)))
  {
    // Just close the dialog without saving or destroying parent
    DebugLog("[%s] Dialog canceled by user\n", PLUGIN_NAME);
    g_dialogOpen = false;
  }
  ImGui::PopStyleColor(3);

  ImGui::End();

  // Rendering
  ImGui::EndFrame();
  g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, FALSE);
  g_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
  g_pd3dDevice->SetRenderState(D3DRS_SCISSORTESTENABLE, FALSE);
  g_pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, g_clearColor, 1.0f, 0);

  if (g_pd3dDevice->BeginScene() >= 0)
  {
    ImGui::Render();
    ImGui_ImplDX9_RenderDrawData(ImGui::GetDrawData());
    g_pd3dDevice->EndScene();
  }

  HRESULT result = g_pd3dDevice->Present(NULL, NULL, NULL, NULL);

  // Handle device loss
  if (result == D3DERR_DEVICELOST && g_pd3dDevice->TestCooperativeLevel() == D3DERR_DEVICENOTRESET)
    ResetDevice();
}

// Create Direct3D device
bool CreateDeviceD3D(HWND hWnd)
{
  if ((g_pD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
    return false;

  // Create the D3DDevice
  ZeroMemory(&g_d3dpp, sizeof(g_d3dpp));
  g_d3dpp.Windowed = TRUE;
  g_d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
  g_d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
  g_d3dpp.EnableAutoDepthStencil = TRUE;
  g_d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
  g_d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE; // Present with vsync

  if (g_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING,
                           &g_d3dpp, &g_pd3dDevice) < 0)
    return false;

  return true;
}

// Cleanup Direct3D resources
void CleanupDeviceD3D()
{
  if (g_pd3dDevice)
  {
    g_pd3dDevice->Release();
    g_pd3dDevice = NULL;
  }
  if (g_pD3D)
  {
    g_pD3D->Release();
    g_pD3D = NULL;
  }
}

// Reset Direct3D device
void ResetDevice()
{
  ImGui_ImplDX9_InvalidateDeviceObjects();
  HRESULT hr = g_pd3dDevice->Reset(&g_d3dpp);
  if (hr == D3DERR_INVALIDCALL)
    IM_ASSERT(0);
  ImGui_ImplDX9_CreateDeviceObjects();
}

// Win32 message handler
LRESULT WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  if (ImGui_ImplWin32_WndProcHandler(hWnd, msg, wParam, lParam))
    return true;

  switch (msg)
  {
  case WM_SIZE:
    if (g_pd3dDevice != NULL && wParam != SIZE_MINIMIZED)
    {
      g_d3dpp.BackBufferWidth = LOWORD(lParam);
      g_d3dpp.BackBufferHeight = HIWORD(lParam);
      ResetDevice();
    }
    return 0;
  case WM_SYSCOMMAND:
    if ((wParam & 0xfff0) == SC_KEYMENU) // Disable ALT application menu
      return 0;
    break;
  case WM_DESTROY:
    // Only mark the dialog as closed, don't post quit message which could affect parent app
    g_dialogOpen = false;
    return 0;
  case WM_CLOSE:
    // Handle close button - just hide the window and mark it as closed
    g_dialogOpen = false;
    // Don't call DestroyWindow directly, let the message loop handle it
    return 0;
  }
  return DefWindowProc(hWnd, msg, wParam, lParam);
}

// Help window procedure
LRESULT CALLBACK HelpWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    if (ImGui_ImplWin32_WndProcHandler(hWnd, message, wParam, lParam))
        return true;

    switch (message)
    {
    case WM_SIZE:
        if (g_pdHelpD3DDevice != NULL && wParam != SIZE_MINIMIZED)
        {
            g_helpD3dpp.BackBufferWidth = LOWORD(lParam);
            g_helpD3dpp.BackBufferHeight = HIWORD(lParam);
            // Reset device
            ImGui_ImplDX9_InvalidateDeviceObjects();
            HRESULT hr = g_pdHelpD3DDevice->Reset(&g_helpD3dpp);
            if (hr == D3DERR_INVALIDCALL)
                IM_ASSERT(0);
            ImGui_ImplDX9_CreateDeviceObjects();
        }
        return 0;
    case WM_CLOSE:
        g_HelpWindowOpen = false;
        ShowWindow(hWnd, SW_HIDE);
        return 0;
    case WM_DESTROY:
        g_HelpWindowOpen = false;
        return 0;
    }

    return CallWindowProc(g_HelpOriginalProc, hWnd, message, wParam, lParam);
}

// Create Direct3D device for help window
bool CreateHelpDeviceD3D(HWND hWnd)
{
    if ((g_pHelpD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
        return false;

    // Create the D3DDevice
    ZeroMemory(&g_helpD3dpp, sizeof(g_helpD3dpp));
    g_helpD3dpp.Windowed = TRUE;
    g_helpD3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    g_helpD3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
    g_helpD3dpp.EnableAutoDepthStencil = TRUE;
    g_helpD3dpp.AutoDepthStencilFormat = D3DFMT_D16;
    g_helpD3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE; // Present with vsync

    if (g_pHelpD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING,
                           &g_helpD3dpp, &g_pdHelpD3DDevice) < 0)
        return false;

    return true;
}

// Cleanup Direct3D resources for help window
void CleanupHelpDeviceD3D()
{
    if (g_pdHelpD3DDevice)
    {
        g_pdHelpD3DDevice->Release();
        g_pdHelpD3DDevice = NULL;
    }
    if (g_pHelpD3D)
    {
        g_pHelpD3D->Release();
        g_pHelpD3D = NULL;
    }
}

// Show the help window using ImGui
void ShowImGuiHelpWindow(HWND hWndParent)
{
    if (g_HelpWindowOpen)
    {
        // If already open, just bring to front
        if (g_HelpWindow)
        {
            SetForegroundWindow(g_HelpWindow);
            return;
        }
    }

    g_HelpWindowParent = hWndParent;
    g_HelpWindowOpen = true;

    // Create window if it doesn't exist
    if (!g_HelpWindow)
    {
        // Register window class
        WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, DefWindowProc, 0L, 0L,
                          GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                          "InstrLoggerHelpClass", NULL };
        RegisterClassEx(&wc);

        // Create window
        g_HelpWindow = CreateWindow(wc.lpszClassName, "Instruction Logger Help",
                                   WS_OVERLAPPEDWINDOW, 100, 100, 600, 500,
                                   NULL, NULL, wc.hInstance, NULL);

        // Initialize Direct3D
        if (!CreateHelpDeviceD3D(g_HelpWindow))
        {
            CleanupHelpDeviceD3D();
            DestroyWindow(g_HelpWindow);
            g_HelpWindow = NULL;
            UnregisterClass("InstrLoggerHelpClass", GetModuleHandle(NULL));
            return;
        }

        // Setup ImGui context
        IMGUI_CHECKVERSION();
        ImGui::CreateContext();
        ImGuiIO &io = ImGui::GetIO();
        io.ConfigFlags |= ImGuiConfigFlags_NavEnableKeyboard;

        // Setup angel-like light ImGui style
        ImGui::StyleColorsLight();
        ImGuiStyle& style = ImGui::GetStyle();

        // Светлая ангельская цветовая гамма
        ImVec4* colors = style.Colors;
        colors[ImGuiCol_Text]                   = ImVec4(0.20f, 0.20f, 0.30f, 1.00f);
        colors[ImGuiCol_TextDisabled]           = ImVec4(0.60f, 0.60f, 0.70f, 1.00f);
        colors[ImGuiCol_WindowBg]               = ImVec4(0.96f, 0.96f, 0.99f, 1.00f);
        colors[ImGuiCol_ChildBg]                = ImVec4(0.92f, 0.94f, 0.98f, 1.00f);
        colors[ImGuiCol_PopupBg]                = ImVec4(0.95f, 0.95f, 0.98f, 0.98f);
        colors[ImGuiCol_Border]                 = ImVec4(0.80f, 0.82f, 0.90f, 0.50f);
        colors[ImGuiCol_BorderShadow]           = ImVec4(0.00f, 0.00f, 0.00f, 0.00f);
        colors[ImGuiCol_FrameBg]                = ImVec4(0.85f, 0.87f, 0.95f, 1.00f);
        colors[ImGuiCol_FrameBgHovered]         = ImVec4(0.75f, 0.80f, 0.93f, 0.60f);
        colors[ImGuiCol_FrameBgActive]          = ImVec4(0.70f, 0.75f, 0.88f, 0.80f);
        colors[ImGuiCol_TitleBg]                = ImVec4(0.75f, 0.80f, 0.95f, 1.00f);
        colors[ImGuiCol_TitleBgActive]          = ImVec4(0.70f, 0.78f, 0.94f, 1.00f);
        colors[ImGuiCol_TitleBgCollapsed]       = ImVec4(0.80f, 0.85f, 0.95f, 0.75f);
        colors[ImGuiCol_MenuBarBg]              = ImVec4(0.85f, 0.87f, 0.97f, 1.00f);
        colors[ImGuiCol_ScrollbarBg]            = ImVec4(0.90f, 0.92f, 0.97f, 1.00f);
        colors[ImGuiCol_ScrollbarGrab]          = ImVec4(0.70f, 0.75f, 0.88f, 0.80f);
        colors[ImGuiCol_ScrollbarGrabHovered]   = ImVec4(0.60f, 0.68f, 0.85f, 0.80f);
        colors[ImGuiCol_ScrollbarGrabActive]    = ImVec4(0.55f, 0.62f, 0.83f, 1.00f);
        colors[ImGuiCol_CheckMark]              = ImVec4(0.50f, 0.60f, 0.85f, 1.00f);
        colors[ImGuiCol_SliderGrab]             = ImVec4(0.60f, 0.70f, 0.90f, 0.80f);
        colors[ImGuiCol_SliderGrabActive]       = ImVec4(0.50f, 0.60f, 0.85f, 1.00f);
        colors[ImGuiCol_Button]                 = ImVec4(0.75f, 0.80f, 0.93f, 0.80f);
        colors[ImGuiCol_ButtonHovered]          = ImVec4(0.65f, 0.73f, 0.90f, 1.00f);
        colors[ImGuiCol_ButtonActive]           = ImVec4(0.55f, 0.65f, 0.88f, 1.00f);
        colors[ImGuiCol_Header]                 = ImVec4(0.70f, 0.78f, 0.94f, 0.60f);
        colors[ImGuiCol_HeaderHovered]          = ImVec4(0.60f, 0.70f, 0.90f, 0.80f);
        colors[ImGuiCol_HeaderActive]           = ImVec4(0.55f, 0.65f, 0.88f, 1.00f);
        colors[ImGuiCol_Separator]              = ImVec4(0.75f, 0.80f, 0.90f, 0.50f);
        colors[ImGuiCol_SeparatorHovered]       = ImVec4(0.65f, 0.70f, 0.85f, 0.70f);
        colors[ImGuiCol_SeparatorActive]        = ImVec4(0.55f, 0.65f, 0.85f, 0.90f);
        colors[ImGuiCol_ResizeGrip]             = ImVec4(0.60f, 0.70f, 0.90f, 0.25f);
        colors[ImGuiCol_ResizeGripHovered]      = ImVec4(0.50f, 0.60f, 0.85f, 0.67f);
        colors[ImGuiCol_ResizeGripActive]       = ImVec4(0.45f, 0.55f, 0.80f, 0.95f);
        colors[ImGuiCol_Tab]                    = ImVec4(0.75f, 0.80f, 0.93f, 0.80f);
        colors[ImGuiCol_TabHovered]             = ImVec4(0.60f, 0.70f, 0.90f, 1.00f);
        colors[ImGuiCol_TabActive]              = ImVec4(0.68f, 0.75f, 0.92f, 1.00f);
        colors[ImGuiCol_TabUnfocused]           = ImVec4(0.80f, 0.85f, 0.95f, 0.97f);
        colors[ImGuiCol_TabUnfocusedActive]     = ImVec4(0.75f, 0.80f, 0.93f, 1.00f);
        colors[ImGuiCol_PlotLines]              = ImVec4(0.40f, 0.45f, 0.70f, 1.00f);
        colors[ImGuiCol_PlotLinesHovered]       = ImVec4(0.30f, 0.35f, 0.60f, 1.00f);
        colors[ImGuiCol_PlotHistogram]          = ImVec4(0.40f, 0.45f, 0.70f, 1.00f);
        colors[ImGuiCol_PlotHistogramHovered]   = ImVec4(0.30f, 0.35f, 0.60f, 1.00f);
        colors[ImGuiCol_TextSelectedBg]         = ImVec4(0.70f, 0.75f, 0.88f, 0.50f);
        colors[ImGuiCol_DragDropTarget]         = ImVec4(0.65f, 0.70f, 0.85f, 0.90f);
        colors[ImGuiCol_NavHighlight]           = ImVec4(0.70f, 0.80f, 0.95f, 1.00f);
        colors[ImGuiCol_NavWindowingHighlight]  = ImVec4(0.60f, 0.70f, 0.90f, 1.00f);
        colors[ImGuiCol_NavWindowingDimBg]      = ImVec4(0.20f, 0.25f, 0.35f, 0.50f);
        colors[ImGuiCol_ModalWindowDimBg]       = ImVec4(0.20f, 0.25f, 0.35f, 0.35f);

        // Изменение стиля элементов для более мягкого, ангельского вида
        style.WindowRounding = 8.0f;
        style.ChildRounding = 4.0f;
        style.FrameRounding = 6.0f;
        style.PopupRounding = 6.0f;
        style.ScrollbarRounding = 6.0f;
        style.GrabRounding = 6.0f;
        style.TabRounding = 6.0f;
        style.FrameBorderSize = 0.0f;
        style.WindowBorderSize = 1.0f;
        style.PopupBorderSize = 1.0f;
        style.ItemSpacing = ImVec2(8, 6);
        style.ItemInnerSpacing = ImVec2(6, 6);
        style.FramePadding = ImVec2(6, 3);
        style.WindowPadding = ImVec2(12, 12);
        style.Alpha = 1.0f;

        // Увеличим размер шрифта для заголовков и кнопок
        io.FontGlobalScale = 1.1f;

        // Использование скругленных краев для всех элементов
        style.WindowTitleAlign = ImVec2(0.5f, 0.5f); // Центрирование заголовка окна
        style.ButtonTextAlign = ImVec2(0.5f, 0.5f);  // Центрирование текста на кнопках

        // Setup Platform/Renderer backends
        ImGui_ImplWin32_Init(g_HelpWindow);
        ImGui_ImplDX9_Init(g_pdHelpD3DDevice);

        // Subclass window procedure
        g_HelpOriginalProc = (WNDPROC)SetWindowLongPtr(g_HelpWindow, GWLP_WNDPROC,
                                                      (LONG_PTR)HelpWndProc);
    }

    // Show window
    ShowWindow(g_HelpWindow, SW_SHOWDEFAULT);
    UpdateWindow(g_HelpWindow);
    SetForegroundWindow(g_HelpWindow);

    // Start message loop for help window
    MSG msg;
    ZeroMemory(&msg, sizeof(msg));
    while (g_HelpWindowOpen && GetMessage(&msg, NULL, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);

        // Render ImGui frame
        if (g_HelpWindowOpen)
        {
            RenderImGuiHelpWindow();
        }
    }

    // Hide window when done
    ShowWindow(g_HelpWindow, SW_HIDE);

    // Cleanup
    ImGui_ImplDX9_Shutdown();
    ImGui_ImplWin32_Shutdown();
    ImGui::DestroyContext();
    CleanupHelpDeviceD3D();
    DestroyWindow(g_HelpWindow);
    g_HelpWindow = NULL;
    UnregisterClass("InstrLoggerHelpClass", GetModuleHandle(NULL));
}

// Render a frame of the help window
void RenderImGuiHelpWindow()
{
    // Start the ImGui frame
    ImGui_ImplDX9_NewFrame();
    ImGui_ImplWin32_NewFrame();
    ImGui::NewFrame();

    // Create a full-window help content
    ImGui::SetNextWindowPos(ImVec2(0, 0));

    // Get window size
    RECT rect;
    GetClientRect(g_HelpWindow, &rect);
    ImGui::SetNextWindowSize(ImVec2((float)(rect.right - rect.left), (float)(rect.bottom - rect.top)));

    ImGui::Begin("Instruction Logger Help", &g_HelpWindowOpen,
                ImGuiWindowFlags_NoResize | ImGuiWindowFlags_NoMove |
                ImGuiWindowFlags_NoCollapse | ImGuiWindowFlags_NoSavedSettings);

    // Get time for all animations
    static float startTime = ImGui::GetTime();
    float time = ImGui::GetTime() - startTime;
    float windowWidth = ImGui::GetWindowWidth();
    float windowHeight = ImGui::GetWindowHeight();
    int frame = (int)(time * 2.0f) % 4; // 2.0f controls animation speed, 4 frames total
    int starFrame = (int)(time * 3.0f) % 3; // Different frequency for stars
    int cloudFrame = (int)(time * 0.7f) % 4; // Slower animation for clouds

    // Draw Star Wars style crawling assembly instructions in the background
    {
        // Store current cursor position to restore later
        ImVec2 originalCursorPos = ImGui::GetCursorPos();

        // Assembly instructions for the background with Star Wars intro text
        static const char* asmInstructions[] = {
            ";---------------------------------------------",
            "; IN A GALAXY.ASM FAR, FAR AWAY...",
            ";---------------------------------------------",
            "; EPISODE 0x64: THE DEBUG AWAKENS",
            ";",
            "; It is a period of binary warfare.",
            "; Rebel programmers, striking from",
            "; hidden terminals, have won their",
            "; first victory against the evil BUG",
            "; EMPIRE.",
            ";",
            "; During the battle, rebel debuggers",
            "; managed to trace the Empire's fatal",
            "; exception and steal execution plans",
            "; to its ultimate weapon, the STACK",
            "; OVERFLOW, a memory corruption with",
            "; enough power to crash an entire",
            "; application...",
            ";---------------------------------------------",
            "MOV EAX, DWORD PTR [ESP+4]   ; Load argument",
            "PUSH EBP                     ; Save base ptr",
            "MOV EBP, ESP                 ; Setup stack frame",
            "SUB ESP, 20h                 ; Allocate local vars",
            "CALL 0x401000                ; Call the force",
            "XOR EDX, EDX                 ; Clear the dark side",
            "JMP SHORT LOC_4010A8         ; Jump to hyperspace",
            "CMP EAX, 1                   ; Check for life",
            "JNZ SHORT LOC_4010B2         ; Not the droids we seek",
            "RET                          ; Return to base",
            "POP EBP                      ; Restore frame",
            "ADD EAX, ECX                 ; Combine forces",
            "TEST AL, 0FFh                ; Test the force",
            "PUSH 0                       ; No fear, no anger",
            "JE SHORT LOC_401030          ; If equal, there is balance",
            "REPE CMPSB                   ; May the force be with you"
        };

        // Number of lines to show
        const int numLines = sizeof(asmInstructions) / sizeof(asmInstructions[0]);

        // Calculate slow scrolling offset
        float scrollOffset = fmodf(time * 15.0f, (float)(numLines * 30 + windowHeight));

        // Draw text at an angle like Star Wars intro
        for (int i = 0; i < numLines; i++) {
            // Calculate position with perspective effect
            float y = windowHeight + i * 30 - scrollOffset;

            // Skip if not visible
            if (y < -30 || y > windowHeight + 30)
                continue;

            // Scale based on perspective (smaller as it goes up)
            float scale = 0.7f + (y / windowHeight) * 0.3f;

            // Calculate width and center horizontally
            float width = strlen(asmInstructions[i]) * 8.0f * scale;
            float x = (windowWidth - width) / 2.0f;

            // Calculate alpha (fade in/out at top and bottom)
            float alpha = 0.08f; // Very faint base alpha
            if (y < 100)
                alpha *= y / 100.0f; // Fade out at top
            else if (y > windowHeight - 100)
                alpha *= (windowHeight - y) / 100.0f; // Fade out at bottom

            // Position cursor and draw text with perspective
            ImGui::SetCursorPos(ImVec2(x, y));
            ImGui::SetWindowFontScale(scale);
            ImGui::TextColored(ImVec4(0.6f, 0.7f, 0.9f, alpha), "%s", asmInstructions[i]);
        }

        // Restore original scale and cursor position
        ImGui::SetWindowFontScale(1.0f);
        ImGui::SetCursorPos(originalCursorPos);
    }

    // Add a title
    ImGui::PushFont(ImGui::GetIO().Fonts->Fonts[0]); // Use default font
    ImGui::TextColored(ImVec4(0.2f, 0.6f, 1.0f, 1.0f), "Instruction Logger Plugin Help");
    ImGui::PopFont();
    ImGui::Separator();

    // Overview section
    ImGui::TextColored(ImVec4(0.9f, 0.6f, 0.3f, 1.0f), "Overview");
    ImGui::TextWrapped(
        "The Instruction Logger plugin allows you to log and analyze specific types of "
        "instructions during debugging sessions. It's particularly useful for tracking "
        "program flow and understanding function call patterns."
    );
    ImGui::Spacing();

    // Features section
    ImGui::TextColored(ImVec4(0.9f, 0.6f, 0.3f, 1.0f), "Instruction Types Logged");
    ImGui::Columns(2, "instruction_types");
    ImGui::Separator();
    ImGui::Text("Instruction"); ImGui::NextColumn();
    ImGui::Text("Description"); ImGui::NextColumn();
    ImGui::Separator();

    ImGui::Text("CALL"); ImGui::NextColumn();
    ImGui::TextWrapped("Function calls - logs when the program calls a function"); ImGui::NextColumn();

    ImGui::Text("JMP"); ImGui::NextColumn();
    ImGui::TextWrapped("Jumps - logs when the program execution jumps to a different location"); ImGui::NextColumn();

    ImGui::Text("RET"); ImGui::NextColumn();
    ImGui::TextWrapped("Returns - logs when a function returns to its caller"); ImGui::NextColumn();

    ImGui::Columns(1);
    ImGui::Separator();
    ImGui::Spacing();

    // Menu options section
    ImGui::TextColored(ImVec4(0.9f, 0.6f, 0.3f, 1.0f), "Menu Options");
    ImGui::Columns(2, "menu_options");
    ImGui::Separator();
    ImGui::Text("Option"); ImGui::NextColumn();
    ImGui::Text("Description"); ImGui::NextColumn();
    ImGui::Separator();

    ImGui::Text("▶ Start Logging"); ImGui::NextColumn();
    ImGui::TextWrapped("Begin recording instructions according to your settings"); ImGui::NextColumn();

    ImGui::Text("■ Stop Logging"); ImGui::NextColumn();
    ImGui::TextWrapped("Stop recording instructions"); ImGui::NextColumn();

    ImGui::Text("🗑️ Clear Log"); ImGui::NextColumn();
    ImGui::TextWrapped("Clear the instruction log"); ImGui::NextColumn();

    ImGui::Text("💾 Save Log..."); ImGui::NextColumn();
    ImGui::TextWrapped("Save the instruction log to a file"); ImGui::NextColumn();

    ImGui::Text("⚙️ Settings..."); ImGui::NextColumn();
    ImGui::TextWrapped("Configure logging options"); ImGui::NextColumn();

    ImGui::Columns(1);
    ImGui::Separator();
    ImGui::Spacing();

    // Settings section
    ImGui::TextColored(ImVec4(0.9f, 0.6f, 0.3f, 1.0f), "Settings Options");
    ImGui::TextWrapped(
        "The Settings dialog allows you to customize which instructions are logged and how:"
    );
    ImGui::Spacing();
    ImGui::BulletText("Log CALL instructions - Track function calls");
    ImGui::BulletText("Log JMP instructions - Track jumps in code execution");
    ImGui::BulletText("Log RET instructions - Track function returns");
    ImGui::BulletText("Log only current module - Restrict logging to the current module");
    ImGui::BulletText("Follow in CPU - Automatically follow execution in the CPU view");
    ImGui::BulletText("Log to file - Save log to a file in addition to displaying in x64dbg");
    ImGui::BulletText("Trace step count - Number of steps to trace when logging starts");
    ImGui::Spacing();

    // Settings storage section
    ImGui::TextColored(ImVec4(0.9f, 0.6f, 0.3f, 1.0f), "Settings Storage");

    // Get INI file path
    char pluginPath[MAX_PATH] = "";
    std::string iniPath = "Unknown Location";

    if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
        GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
    {
        char *lastSlash = strrchr(pluginPath, '\\');
        if (lastSlash)
        {
            *(lastSlash + 1) = 0; // Truncate to directory
            iniPath = std::string(pluginPath) + "InstrLoggerSettings.ini";
        }
    }

    ImGui::TextWrapped(
        "Settings are automatically saved to and loaded from:"
    );
    ImGui::TextColored(ImVec4(0.5f, 0.8f, 0.5f, 1.0f), "%s", iniPath.c_str());
    ImGui::TextWrapped(
        "This allows your preferences to persist between x64dbg sessions."
    );
    ImGui::Spacing();

    // Tips section
    ImGui::TextColored(ImVec4(0.9f, 0.6f, 0.3f, 1.0f), "Tips");
    ImGui::BulletText("Start a debugging session before using the logger");
    ImGui::BulletText("Use 'Log only current module' to reduce noise from system DLLs");
    ImGui::BulletText("The trace file is automatically saved when logging stops");
    ImGui::BulletText("Adjust the trace step count based on your analysis needs");
    ImGui::Spacing();

    // ASCII cat animation with additional art elements
    ImGui::Separator();
    ImGui::Spacing();

    // Use the variables already defined above for animation
    float catWidth = 120.0f;

    // Draw stars (top left)
    ImGui::SetCursorPos(ImVec2(windowWidth * 0.2f, ImGui::GetCursorPosY()));
    switch(starFrame) {
        case 0: ImGui::TextColored(ImVec4(0.9f, 0.9f, 0.2f, 0.8f), "✧ ⋆"); break;
        case 1: ImGui::TextColored(ImVec4(0.9f, 0.9f, 0.2f, 0.9f), "⋆ ✧"); break;
        case 2: ImGui::TextColored(ImVec4(0.9f, 0.9f, 0.2f, 1.0f), "✦ ✧"); break;
    }

    // Draw moon (top right)
    ImGui::SetCursorPos(ImVec2(windowWidth * 0.8f, ImGui::GetCursorPosY()));
    if (frame % 2 == 0)
        ImGui::TextColored(ImVec4(0.8f, 0.8f, 0.9f, 0.9f), "🌙");
    else
        ImGui::TextColored(ImVec4(0.8f, 0.8f, 0.9f, 1.0f), "🌙");

    // Draw cloud (moving)
    float cloudX = (windowWidth * 0.1f) + (sin(time * 0.5f) * windowWidth * 0.1f);
    ImGui::SetCursorPos(ImVec2(cloudX, ImGui::GetCursorPosY() + 10));
    switch(cloudFrame) {
        case 0: ImGui::TextColored(ImVec4(0.9f, 0.9f, 0.9f, 0.7f), "~ ~ ~"); break;
        case 1: ImGui::TextColored(ImVec4(0.9f, 0.9f, 0.9f, 0.8f), "~ ~ ~"); break;
        case 2: ImGui::TextColored(ImVec4(0.9f, 0.9f, 0.9f, 0.9f), "~ ~ ~"); break;
        case 3: ImGui::TextColored(ImVec4(0.9f, 0.9f, 0.9f, 0.8f), "~ ~ ~"); break;
    }

    // Position for the cat (center)
    ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);

    // Different cat frames
    switch(frame) {
        case 0:
            ImGui::Text("/\\_/\\");
            ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
            ImGui::Text("(>ω<)");
            ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
            ImGui::Text("/ >  \\");
            break;
        case 1:
            ImGui::Text("/\\_/\\");
            ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
            ImGui::Text("(^ω^)");
            ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
            ImGui::Text("/ >  \\");
            break;
        case 2:
            ImGui::Text("/\\_/\\");
            ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
            ImGui::Text("(=ω=)");
            ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
            ImGui::Text("/ >  \\");
            break;
        case 3:
            ImGui::Text("/\\_/\\");
            ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
            ImGui::Text("(·ω·)");
            ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
            ImGui::Text("/ >  \\");
            break;
    }

    // Draw flowers (bottom)
    ImGui::SetCursorPos(ImVec2(windowWidth * 0.3f, ImGui::GetCursorPosY() + 5));
    ImGui::TextColored(ImVec4(0.9f, 0.5f, 0.7f, 0.8f + 0.2f * sin(time * 1.5f)), "✿");

    ImGui::SetCursorPos(ImVec2(windowWidth * 0.7f, ImGui::GetCursorPosY()));
    ImGui::TextColored(ImVec4(0.7f, 0.9f, 0.5f, 0.8f + 0.2f * sin(time * 1.8f + 1.0f)), "❀");

    // Draw a small bird
    float birdX = (windowWidth * 0.6f) + (sin(time * 0.8f) * windowWidth * 0.1f);
    ImGui::SetCursorPos(ImVec2(birdX, ImGui::GetCursorPosY() - 30));
    if (frame % 2 == 0)
        ImGui::TextColored(ImVec4(0.4f, 0.7f, 0.9f, 0.9f), ">");
    else
        ImGui::TextColored(ImVec4(0.4f, 0.7f, 0.9f, 0.9f), "^");

    // Add wasm.in domain with fade effect
    ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
    float alpha = 0.3f + 0.25f * sinf(time * 0.5f);
    ImVec2 curPos = ImGui::GetCursorPos();
    ImGui::SetCursorPos(ImVec2(curPos.x + 2, curPos.y + 2));
    ImGui::TextColored(ImVec4(0.0f, 0.0f, 0.0f, alpha), "wasm.in");

    ImGui::SetCursorPosX((windowWidth - catWidth) * 0.5f);
    ImGui::TextColored(ImVec4(0.5f, 0.5f, 0.9f, alpha + 0.3f), "wasm.in");

    // Add animated floating assembly instruction display before the Star Wars quote
    ImGui::Spacing();
    ImGui::Spacing();

    // Create a fixed display area for the floating instructions
    float displayAreaHeight = 100.0f;
    float displayAreaWidth = windowWidth * 0.8f;
    ImVec2 displayAreaPos = ImVec2((windowWidth - displayAreaWidth) * 0.5f, ImGui::GetCursorPosY());

    // Draw a subtle background for the display area
    ImGui::SetCursorPos(displayAreaPos);
    ImGui::GetWindowDrawList()->AddRectFilled(
        ImVec2(displayAreaPos.x, displayAreaPos.y),
        ImVec2(displayAreaPos.x + displayAreaWidth, displayAreaPos.y + displayAreaHeight),
        ImGui::GetColorU32(ImVec4(0.2f, 0.2f, 0.3f, 0.1f)),
        8.0f
    );

    // Assembly instructions for the display
    static const char* displayInstructions[] = {
        "MOV EAX, DWORD PTR [ESP+4]   ; Load argument",
        "PUSH EBP                     ; Save base ptr",
        "MOV EBP, ESP                 ; Setup stack frame",
        "SUB ESP, 20h                 ; Allocate local vars",
        "CALL 0x401000                ; Call the force",
        "XOR EDX, EDX                 ; Clear the dark side",
        "JMP SHORT LOC_4010A8         ; Jump to hyperspace",
        "CMP EAX, 1                   ; Check for life",
        "JNZ SHORT LOC_4010B2         ; Not the droids we seek"
    };

    const int numDisplayLines = sizeof(displayInstructions) / sizeof(displayInstructions[0]);

    // Display multiple lines with staggered animations
    for (int i = 0; i < 3; i++) {
        // Calculate which instruction to show (cycling through the array)
        int instructionIndex = (int)(time * 0.3f + i * 2.5f) % numDisplayLines;

        // Calculate Y position with floating animation
        float baseY = displayAreaPos.y + 10.0f + (i * 30.0f);
        float offsetY = sin(time * 0.8f + i * 1.0f) * 5.0f;
        float y = baseY + offsetY;

        // Calculate alpha for fade in/out effect
        float alpha = 0.7f + 0.3f * sin(time * 0.5f + i * 0.7f);

        // Draw the instruction text with a slight glow effect
        ImGui::SetCursorPos(ImVec2(displayAreaPos.x + 10.0f, y + 1.0f));
        ImGui::TextColored(ImVec4(0.3f, 0.5f, 0.9f, alpha * 0.3f), "%s", displayInstructions[instructionIndex]);

        ImGui::SetCursorPos(ImVec2(displayAreaPos.x + 10.0f, y));
        ImGui::TextColored(ImVec4(0.3f, 0.6f, 1.0f, alpha), "%s", displayInstructions[instructionIndex]);
    }

    // Reset cursor position after the display area
    ImGui::SetCursorPos(ImVec2(ImGui::GetCursorPosX(), displayAreaPos.y + displayAreaHeight + 10.0f));

    // Star Wars quote
    ImGui::Spacing();

    // Calculate different fade timing for this text
    float textAlpha = 0.2f + 0.2f * sinf(time * 0.3f - 0.5f);

    // Draw shadow for Star Wars reference text
    ImGui::SetCursorPosX((windowWidth - 240) * 0.5f);
    curPos = ImGui::GetCursorPos();
    ImGui::SetCursorPos(ImVec2(curPos.x + 2, curPos.y + 2));
    ImGui::TextColored(ImVec4(0.0f, 0.0f, 0.0f, textAlpha), "May the debug be with you...");

    // Draw main Star Wars reference text
    ImGui::SetCursorPosX((windowWidth - 240) * 0.5f);
    ImGui::TextColored(ImVec4(0.6f, 0.7f, 1.0f, textAlpha + 0.2f), "May the debug be with you...");

    ImGui::Spacing();

    // Close button at the bottom
    ImGui::SetCursorPosX((ImGui::GetWindowWidth() - 120) * 0.5f);
    if (ImGui::Button("Close", ImVec2(120, 0)))
    {
        g_HelpWindowOpen = false;
    }

    ImGui::End();

    // Rendering
    ImGui::EndFrame();
    g_pdHelpD3DDevice->SetRenderState(D3DRS_ZENABLE, FALSE);
    g_pdHelpD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
    g_pdHelpD3DDevice->SetRenderState(D3DRS_SCISSORTESTENABLE, FALSE);
    g_pdHelpD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, g_clearColor, 1.0f, 0);

    if (g_pdHelpD3DDevice->BeginScene() >= 0)
    {
        ImGui::Render();
        ImGui_ImplDX9_RenderDrawData(ImGui::GetDrawData());
        g_pdHelpD3DDevice->EndScene();
    }

    HRESULT result = g_pdHelpD3DDevice->Present(NULL, NULL, NULL, NULL);

    // Handle device loss
    if (result == D3DERR_DEVICELOST && g_pdHelpD3DDevice->TestCooperativeLevel() == D3DERR_DEVICENOTRESET)
    {
        ImGui_ImplDX9_InvalidateDeviceObjects();
        HRESULT hr = g_pdHelpD3DDevice->Reset(&g_helpD3dpp);
        if (hr == D3DERR_INVALIDCALL)
            IM_ASSERT(0);
        ImGui_ImplDX9_CreateDeviceObjects();
    }
}
