#pragma once

#include <cstdint>
#include <windows.h>
#include <winternl.h>
#include "../pluginsdk/_plugins.h"

// Plugin name and version
#define PLUGIN_NAME "InstrLogger"
#define PLUGIN_VERSION 1

// Settings INI filename
#define CUSTOM_INI_FILENAME "InstrLoggerSettings.ini"

// Menu IDs
#define MENU_START_LOGGING 1
#define MENU_STOP_LOGGING 2
#define MENU_CLEAR_LOG 3
#define MENU_SAVE_LOG 4
#define MENU_SETTINGS 5
#define MENU_HELP 6

// Settings dialog IDs
#define IDC_CALLS_CHECK 1001
#define IDC_JUMPS_CHECK 1002
#define IDC_RETURNS_CHECK 1003
#define IDC_MODULE_CHECK 1004
#define IDC_FOLLOW_CHECK 1005
#define IDC_FILE_CHECK 1006
#define IDC_FILE_PATH 1007
#define IDC_BROWSE_BTN 1008
#define IDC_OK_BTN 1009
#define IDC_CANCEL_BTN 1010

// Exported plugin functions
extern "C" __declspec(dllexport) bool pluginit(PLUG_INITSTRUCT* initStruct);
extern "C" __declspec(dllexport) void plugstop();
extern "C" __declspec(dllexport) void plugsetup(PLUG_SETUPSTRUCT* setupStruct);
extern "C" __declspec(dllexport) void CBBREAKPOINT(CBTYPE cbType, PLUG_CB_BREAKPOINT* info);
extern "C" __declspec(dllexport) void CBSTEPPED(CBTYPE cbType, PLUG_CB_STEPPED* info);
extern "C" __declspec(dllexport) void CBTRACEEXECUTE(CBTYPE cbType, PLUG_CB_TRACEEXECUTE* info);
extern "C" __declspec(dllexport) void CBMENUENTRY(CBTYPE cbType, PLUG_CB_MENUENTRY* info);
extern "C" __declspec(dllexport) void CBWINEVENT(CBTYPE cbType, PLUG_CB_WINEVENT* info);

// New functions

// Helper to check if instruction is a jump
inline bool IsJumpInstruction(const char* instruction)
{
	// Check for exact "jmp" instruction
	if (strncmp(instruction, "jmp", 3) == 0 && (instruction[3] == ' ' || instruction[3] == '\t' || instruction[3] == '\0'))
		return true;

	// Check for conditional jumps (j*)
	if (instruction[0] == 'j' &&
		instruction[1] != 'm' &&
		instruction[1] != '\0' &&
		(instruction[2] == ' ' || instruction[2] == '\t' || instruction[2] == 'c' || instruction[2] == 'n' ||
			instruction[2] == 'e' || instruction[2] == 'b' || instruction[2] == 'a' || instruction[2] == 'g' ||
			instruction[2] == 'l' || instruction[2] == 'p' || instruction[2] == 's' || instruction[2] == 'o'))
		return true;

	return false;
}

// Helper to check if instruction is a call
inline bool IsCallInstruction(const char* instruction)
{
	// Check for exact "call" instruction at the beginning
	if (strncmp(instruction, "call", 4) == 0 && (instruction[4] == ' ' || instruction[4] == '\t' || instruction[4] == '\0'))
		return true;

	return false;
}

// Helper to check if instruction is a return
inline bool IsRetInstruction(const char* instruction)
{
	// Check for exact "ret" instruction
	if (strncmp(instruction, "ret", 3) == 0 && (instruction[3] == ' ' || instruction[3] == '\t' || instruction[3] == '\0'))
		return true;

	return false;
}

